#include <iostream>
#include <vector>
#include "svg.h"

void svg_begin(double width, double height)
{
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void svg_end()
{
    std::cout << "</svg>\n";
}

void svg_text(double left, double baseline, std::string text)
{
    std::cout << "<text x='" << left << "' y='"<< baseline <<"'>"<< text <<"</text>";
}

void svg_rect(double x, double y, double width, double height, std::string fill = "black", std::string stroke = "black")
{
    std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << "' fill='" << fill << "' stroke='" << stroke << "' />\n";
}

std::string check_color(const std::string& color)
{
    if (color.find(" ") == -1 || color.find('#') == 0)
    {
        return color;
    }
    else
    {
        return "black";
    }
}



void show_histogram_svg(const std::vector<std::size_t>& bins, const std::vector<std::string>& colors)
{

    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;

    const std::size_t MAX_ASTERISK = IMAGE_WIDTH - TEXT_WIDTH;
    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);
    double top = 0;

     std::size_t maxbin=bins[0];
    for (std::size_t i=0; i < bins.size(); i++){
        if (maxbin < bins[i]){
            maxbin=bins[i];
        }
    }


    if (maxbin<=76) {
        for (std::size_t j=0; j < bins.size(); j++) {
            const double bin_width= MAX_ASTERISK * (static_cast<double>(bins[j]) / maxbin);
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[j]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT, check_color(colors[j]), "black");
            top += BIN_HEIGHT;
        }
        svg_end();
    }
    else {
        for (std::size_t j=0; j < bins.size(); j++) {
            const double bin_width= MAX_ASTERISK * (static_cast<double>(bins[j]) / maxbin);
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[j]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT, check_color(colors[j]), "black");
            top += BIN_HEIGHT;
        }
        svg_end();
    }
}
